<?php
/**
 * Holler Box Pro Functions
 * @since       0.1.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

if( !class_exists( 'Holler_Sales_Functions' ) ) {

    /**
     * Holler_Sales_Functions class
     *
     * @since       0.2.0
     */
    class Holler_Sales_Functions {

        /**
         * @var         Holler_Sales_Functions $instance The one true Holler_Sales_Functions
         * @since       0.2.0
         */
        private static $instance;
        public static $errorpath = '../php-error-log.php';
        public static $active = array();
        // sample: error_log("meta: " . $meta . "\r\n",3,self::$errorpath);

        /**
         * Get active instance
         *
         * @access      public
         * @since       0.2.0
         * @return      object self::$instance The one true Holler_Sales_Functions
         */
        public static function instance() {
            if( !self::$instance ) {
                self::$instance = new Holler_Sales_Functions();
                self::$instance->hooks();
            }

            return self::$instance;
        }


        /**
         * Include necessary files
         *
         * @access      private
         * @since       0.2.0
         * @return      void
         */
        private function hooks() {

            add_filter( 'hollerbox_content', array( $this, 'maybe_filter_content' ), 10, 2 );

            add_filter( 'hwp_custom_settings_save', array( $this, 'maybe_hide_avatar' ) );

            add_action( 'admin_init', array( $this, 'automatic_updates') );

            add_action( 'admin_init', array( $this, 'maybe_activate_deactivate_license') );

            add_action( 'hwp_type_settings', array( $this, 'type_settings') );

            add_action( 'hwp_license_fields', array( $this, 'license_fields' ) );

            // add_action( 'hwp_after_position_settings', array( $this, 'fomo_settings' ) );

        }

        /**
         * Add fomo setting
         *
         * @access      public
         * @since       0.1
         */
        public function type_settings( $id ) {

            ?>

            <label class="hwp-radio-withimage">
                <span class="text">FOMO</span>
                <img src="<?php echo Holler_Box_Sales_URL . '/assets/img/fomo-icon.png'; ?>" class="hwp-radio-image" />
                <input type="radio" name="hwp_type" value="fomo" <?php checked( "fomo", get_post_meta( $id, 'hwp_type', 1 ) ); ?> />
            </label>

            <?php
        }

        /**
         * Add FOMO settings. Useful for adding MailChimp and other support.
         *
         * @since       0.2.0
         * @return      void
         */
        public function fomo_settings( $post_id ) {
            // Unused, will be useful when there are more fomo choices like MailChimp, etc.
            ?>

            <div class="hwp-section" id="position-settings">

                <h4>
                    <label for="position"><?php _e( 'FOMO Notification Settings' ); ?></label>
                </h4>

                <input type="radio" name="fomo_setting" value="edd" <?php checked( "edd", get_post_meta( $post_id, 'fomo_setting', true ) ); ?> />
                <label>EDD Sale</label>

                <input type="radio" name="fomo_setting" value="woo" <?php checked( "woo", get_post_meta( $post_id, 'fomo_setting', true ) ); ?> />
                <label>WooCommerce Sale</label>

            </div>

            <?php
        }

        /**
         * Check if we should filter the content and add sale notification
         * $box_id is the notification ID, not the current page ID
         *
         * @since       0.2.0
         * @return      void
         */
        public function maybe_filter_content( $content, $box_id ) {

            // Check for type. If it's fomo, filter the content.
            $type = get_post_meta( $box_id, 'hwp_type', 1 );
            
            // hwp_sale_id setting DEPRECATED
            // this is the ID where we should show sales notifications
            $setting_id = get_option( 'hwp_sale_id' );

            // If the old setting id matches, or the type is fomo, filter the content to show sale notification.
            if( !empty( $setting_id ) && $box_id === $setting_id || $type === 'fomo' ) {

                // detect ecomm plugin
                if( class_exists('Easy_Digital_Downloads') ) {

                    return $this->edd_sale( $content );

                } elseif( class_exists('WooCommerce') ) {
                    
                    return $this->woocommerce_sale( $content );

                }

            } else {

                // do nothing
                return $content;

            } 
    
        }

        /**
         * Hook into settings save. If someone creates a FOMO box and saves it, delete the avatar email so it doesn't show.
         *
         * @since       0.2.0
         * @return      void
         */
        public function maybe_hide_avatar( $id ) {

            // Check for type. If it's fomo, delete the avatar.
            $type = get_post_meta( $id, 'hwp_type', 1 );
            if( $type === 'fomo' )
                delete_post_meta( $id, 'avatar_email' );

        }

        /**
         * Add latest EDD sale to Hollerbox
         *
         * @since       0.1.0
         * @return      void
         */
        public function edd_sale( $content ) {

            // transient is used to step through sales so we don't show the same one each time.
            $transient = get_transient( 'hwp_increment' );
            if( $transient < 10 ) {
                $transient = intval( $transient ) + 1;
            } else {
                $transient = 1;
            }

            $edd_payment = get_post_type_object( 'edd_payment' );
            $args = array(
                'post_type' => 'edd_payment',
                'post_status' => array( 'publish' ),
                'posts_per_page' => 1,
                'offset' => intval( $transient ) - 1
            );

            $return = self::edd_payment_loop( $args, $content );

            set_transient( 'hwp_increment', $transient, DAY_IN_SECONDS );

            return $return;

        }

        /**
         * Loop through EDD payments to get a FOMO sale
         *
         * @since       0.3.0
         * @return      void
         */
        public function edd_payment_loop( $args, $content ) {

            // The Query
            $the_query = new WP_Query( $args );

            if ( $the_query->have_posts() ) {
                while ( $the_query->have_posts() ) {
                    $the_query->the_post();

                    $post_id = get_the_ID();
                    $download_meta = get_post_meta( $post_id, '_edd_payment_meta', 1 );
                    $download_id = $download_meta['downloads'][0]['id'];

                    $customer_email = $download_meta['user_info']['email'];

                    $download = edd_get_download( $download_id );
                    $content = get_avatar( $customer_email, 50 );

                    $content .= $download_meta['user_info']['first_name'] . ' purchased <a href="' . get_permalink( $download_id ) . '">' . $download->post_title . '</a>';

                }
                wp_reset_postdata();
            }

            return apply_filters( 'hwp_fomo_content', $content, $post_id );
        }

        /**
         * Add latest Woo sale to Hollerbox
         *
         * @since       0.1.0
         * @return      void
         */
        public function woocommerce_sale( $content ) {

            $args = array( 
                'status' => 'wc-completed',
                'type' => 'shop_order',
                'limit' => 1
             );

            $order = wc_get_orders( $args )[0];
            $id = $order->get_id();

            $items = $order->get_items();

            foreach ( $items as $item ) {
                $name = $item['name'];
                $product_id = $item['product_id'];
            }

            $first_name = $order->get_billing_first_name();
            $email = $order->get_billing_email();

            $content = get_avatar( $email, 50 );
            $content .= $first_name . ' just purchased <a href="' . get_permalink( $product_id ) . '">' . $name . '</a>';

            return $content;

        }

        /**
         * Initialize EDD Software Licensing
         *
         * @access      public
         * @since       0.2
         */
        public function automatic_updates() {

            // retrieve our license key from the DB
            $license_key = trim( get_option( 'hwp_fomo_license' ) );
            // setup the updater
            $edd_updater = new EDD_SL_Plugin_Updater( HWP_SALES_STORE_URL, Holler_Box_Sales_File, array( 
                    'version'   => Holler_Box_Sales_VER,       // current version number
                    'license'   => $license_key,    // license key (used get_option above to retrieve from DB)
                    'item_id'     => 156,    // id of this plugin
                    'author'    => 'Scott Bolinger',  // author of this plugin
                    'url'           => home_url(),
                    'beta'          => false
                )
            );

            // save our license settings
            self::edd_license_save();

        }

        /**
         * Add license key fields
         *
         * @access      public
         * @since       0.3
         */
        public function license_fields() {

            $license_key = get_option( 'hwp_fomo_license' );
            $license_status = get_option( 'hwp_fomo_license_status' );

            $class = '';

            if( $license_status !== false && $license_status == 'valid' ) 
                $class = 'hwp-license-active';

            ?>
            <tr valign="top">
                <th scope="row" valign="top">
                    <?php _e('FOMO license key'); ?>
                </th>
                <td>
                    <input id="hwp_fomo_license" name="hwp_fomo_license" type="text" class="regular-text <?php echo $class; ?>" value="<?php esc_attr_e( $license_key ); ?>" />
                    <?php if( $license_status !== false && $license_status == 'valid' ) echo '<br><small>License active</small>'; ?>
                </td>
            </tr>

            <?php if( false !== $license_key ) : ?>
                <tr valign="top">
                    <th scope="row" valign="top">
                        
                    </th>
                    <td>
                        <?php if( $license_status !== false && $license_status == 'valid' ) : ?>
                            <input type="submit" class="button-secondary" name="fomo_license_deactivate" value="<?php _e('Deactivate License'); ?>"/>
                        <?php else: ?>
                                <input type="submit" class="button-primary" name="fomo_license_activate" value="<?php _e('Activate License'); ?>"/>
                        <?php endif; ?>
                    </td>
                </tr>
            <?php 
            endif;
        }

        /**
         * Save license settings
         *
         * @access      public
         * @since       0.4
         */
        public function edd_license_save() {

            // run a quick security check
            // if ( !isset( $_POST['hollerbox_settings_nonce'] ) || !wp_verify_nonce( $_POST['hollerbox_settings_nonce'], basename( __FILE__ ) ) )
            // return;

            // don't run if we are deactivating licenses
            if( isset( $_POST['fomo_license_deactivate'] ) || isset( $_POST['pro_license_deactivate'] ) )
                return;

            /** Licensing **/
            $license_key = get_option( 'hwp_fomo_license' );
            $license_status = get_option('hwp_fomo_license_status');

            $postval = ( isset( $_POST['hwp_fomo_license'] ) ? $_POST['hwp_fomo_license'] : '' );

            if( $postval != '' ) {

                update_option( 'hwp_fomo_license', sanitize_text_field( $postval ) );

            } 

            if( isset( $_POST['hwp_fomo_license'] ) && $_POST['hwp_fomo_license'] === '' ) {
                // delete license stuff if field is blank
                delete_option( 'hwp_fomo_license' );
                delete_option( 'hwp_fomo_license_status' );

            }

        }

        /**
         * Maybe Deactivate license
         *
         * @access      public
         * @since       0.2
         */
        public function maybe_activate_deactivate_license() {

            // listen for our activate button to be clicked
            if( isset( $_POST['fomo_license_deactivate'] ) ) {

                $license_key = trim( get_option( 'hwp_fomo_license' ) );

                $res = Holler_Pro_Admin::deactivate_license( $license_key, 156 );

                // wp_die( $res );

                // $license_data->license will be either "deactivated" or "failed"
                if( $res == 'deactivated' ) {
                    delete_option( 'hwp_fomo_license_status' );
                    $redirect = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                } else {
                    $base_url = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                    $redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $res ) ), $base_url );
                }

                wp_redirect( $redirect );
            } else if( isset( $_POST['fomo_license_activate'] ) && isset( $_POST['hwp_fomo_license'] ) ) {

                $license_key = $_POST['hwp_fomo_license'];

                $res = Holler_Pro_Admin::activate_license( $license_key, 156 );

                // $res is any error messages from license activation. Display them to user on redirect if necessary.
                if ( ! empty( $res['msg'] ) ) {
                    $base_url = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                    $redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $res['msg'] ) ), $base_url );
                    wp_redirect( $redirect );
                    exit();
                }

                // $license_data->license will be either "valid" or "invalid"
                update_option( 'hwp_fomo_license_status', $res['status'] );

                wp_redirect( admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' ) );

            }
        }

    }

    $holler_sales_functions = new Holler_Sales_Functions();
    $holler_sales_functions->instance();

} // end class_exists check